package org.eclipse.datatools.enablement.sybase.models.sybasesqlmodel.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.datatools.connectivity.sqm.core.containment.ContainmentServiceImpl;
import org.eclipse.datatools.enablement.sybase.models.sybasesqlmodel.SybaseAuthorizedObject;
import org.eclipse.datatools.enablement.sybase.models.sybasesqlmodel.SybasePrivilege;
import org.eclipse.datatools.modelbase.sql.accesscontrol.AuthorizationIdentifier;
import org.eclipse.datatools.modelbase.sql.accesscontrol.Privilege;
import org.eclipse.datatools.modelbase.sql.routines.Routine;
import org.eclipse.datatools.modelbase.sql.schema.Database;
import org.eclipse.datatools.modelbase.sql.schema.SQLObject;
import org.eclipse.datatools.modelbase.sql.tables.Column;
import org.eclipse.datatools.modelbase.sql.tables.Table;

/**
 * Privilege related utilities
 * 
 * @author Dafan Yang
 */
public class PrivilegesUtil
{
    /**
     * Returns all issued privileges of the given SQL object
     * 
     * @param object the SQL object
     * @return all issued privileges
     */
    public static List getSQLObjectPrivileges(SQLObject object)
    {
        List privileges = new ArrayList();
        if (object instanceof Table || object instanceof Routine || object instanceof Column)
        {
            if (object instanceof SybaseAuthorizedObject)
            {
                privileges = ((SybaseAuthorizedObject) object).getPrivileges();
            }
            else if (object instanceof Column && object.eContainer() instanceof SybaseAuthorizedObject)
            {// special handle sybaseasecolumn
                privileges = ((SybaseAuthorizedObject) object.eContainer()).getPrivileges();
            }
        }
        return privileges;
    }

    /**
     * Returns all privileges of the given SQL object owned by the given authid
     * 
     * @param authid
     * @param obj
     * @return
     */
    public static List getReceivedSQLObjectPrivileges(AuthorizationIdentifier authid, SQLObject obj)
    {
        List matchedPrivileges = new ArrayList();
        List privs = new ArrayList();
        if (obj instanceof SybaseAuthorizedObject)
        {
            privs = ((SybaseAuthorizedObject) obj).getPrivileges();
        }
        else if (obj instanceof Column && obj.eContainer() instanceof SybaseAuthorizedObject)
        {// special handle sybaseasecolumn
            privs = ((SybaseAuthorizedObject) obj.eContainer()).getPrivileges();
        }
        // else
        // {
        // privs = authid.getReceivedPrivilege();
        // }
        Iterator iter = privs.iterator();
        while (iter.hasNext())
        {
            SybasePrivilege p = (SybasePrivilege) iter.next();
            if (p.getObject() == obj && p.getGrantee() == authid)
            {
                matchedPrivileges.add(p);
            }
        }
        return matchedPrivileges;
    }

    /**
     * Returns the privilege of the given SQL object owned by the given authid with action specified
     * 
     * @param authid
     * @param obj
     * @param action
     * @return
     */
    public static Privilege getPrivilege(AuthorizationIdentifier authid, SQLObject obj, String action)
    {
        List matchedPrivileges = getReceivedSQLObjectPrivileges(authid, obj);
        Privilege matchedP = null;
        Iterator iter = matchedPrivileges.iterator();
        while (iter.hasNext())
        {
            Privilege p = (Privilege) iter.next();
            if (p.getAction().equals(action))
            {
                matchedP = p;
                break;
            }
        }
        return matchedP;
    }
}
